// Copyright © 2023 ab180. All rights reserved.
#include "AirbridgeAndroidCallback.h"

#if PLATFORM_ANDROID
#include "AirbridgeAndroidCore.h"
#include "AirbridgeUnrealModule.h"
#include "Platforms/Android/Util/AirbridgeJNI.h"

#define AIRBRIDGE_UNREAL_MODULE_POINTER		(FAirbridgeUnrealModule::Get())
#define ANDROID_CORE_POINTER				(static_cast<AirbridgeAndroidCore*>((AIRBRIDGE_UNREAL_MODULE_POINTER)->GetAirbridge().Get()))

jint GetCallbackType(JNIEnv *Env, jobject This)
{
	if (This == nullptr) {
		return -1;
	}
	jclass AirbridgeCallbackClass = Env->GetObjectClass(This);

	jmethodID GetCallbackTypeMethod = Env->GetMethodID(AirbridgeCallbackClass, "getCallbackType", "()I");
	if (GetCallbackTypeMethod == nullptr) {
		return -1;
	}

	jint Type = Env->CallIntMethod(This, GetCallbackTypeMethod);
	return Type;
}

FString GetCallbackUUID(JNIEnv *Env, jobject This)
{
	if (This == nullptr) {
		return "";
	}
	jclass AirbridgeCallbackClass = Env->GetObjectClass(This);

	jmethodID GetUUIDMethod = Env->GetMethodID(AirbridgeCallbackClass, "getUUID", "()Ljava/lang/String;");
	if (GetUUIDMethod == nullptr) {
		return "";
	}

	jstring UUID = (jstring) Env->CallObjectMethod(This, GetUUIDMethod);
	return ConvertString(Env, UUID);
}

bool IsBroadcastCallback(JNIEnv *Env, jobject This)
{
	if (This == nullptr) {
		return false;
	}
	jclass AirbridgeCallbackClass = Env->GetObjectClass(This);

	jmethodID IsBroadcastMethod = Env->GetMethodID(AirbridgeCallbackClass, "isBroadcast", "()Z");
	if (IsBroadcastMethod == nullptr) {
		return false;
	}

	jboolean Result = Env->CallBooleanMethod(This, IsBroadcastMethod);
	return (Result != JNI_FALSE);
}

FString ConvertString(JNIEnv *Env, jstring String)
{
	if (String == nullptr) {
		return "";
	}

	// Copies the new key into its final C string buffer
	const char* UTFString = Env->GetStringUTFChars(String, nullptr);
	if (UTFString == nullptr) {
		return "";  /* OutOfMemoryError already thrown */
	}
	
	FString ToReturn(UTF8_TO_TCHAR(UTFString));

	// Release heap memory
	Env->ReleaseStringUTFChars(String, UTFString);
	Env->DeleteLocalRef(String);

	return ToReturn;
}

FString GetUriString(JNIEnv *Env, jobject Uri)
{
	if (Uri == nullptr) {
		return "";
	}
	jclass UriClass = Env->GetObjectClass(Uri);

	jmethodID ToStringMethod = Env->GetMethodID(UriClass, "toString", "()Ljava/lang/String;");
	if (ToStringMethod == nullptr) {
		return "";
	}

	jstring String = (jstring) Env->CallObjectMethod(Uri, ToStringMethod);
	return ConvertString(Env, String);
}

extern "C" {
	/*
	 * Class:     co_ab180_airbridge_ue_callback_AirbridgeOnSuccessCallback
	 * Method:    invoke
	 * Signature: (Ljava/lang/Object;)V
	 */
	JNIEXPORT void JNICALL Java_co_ab180_airbridge_ue_callback_AirbridgeOnSuccessCallback_invoke
	(JNIEnv *Env, jobject This, jobject Result)
	{
		FString UUID = GetCallbackUUID(Env, This);				// Get Callback UUID
		bool IsBroadcast = IsBroadcastCallback(Env, This);		// Is Broadcast
		
		switch (GetCallbackType(Env, This)) {
		case HandleDeeplink:
			if (Result == nullptr) { return; }	// [ Nullable ]
			// Check if module is loaded
			if (AIRBRIDGE_UNREAL_MODULE_POINTER)
			{
				ANDROID_CORE_POINTER->ProcessHandleDeeplink(GetUriString(Env, Result));
			}
			// If module is not loaded
			else
			{
				AirbridgeAndroidCore::StartDeeplink = GetUriString(Env, Result);
			}
			break;
		case HandleDeferredDeeplink:
			if (Result == nullptr) { return; }	// [ Nullable ]
			if (AIRBRIDGE_UNREAL_MODULE_POINTER)
				ANDROID_CORE_POINTER->ProcessHandleDeferredDeeplink(GetUriString(Env, Result));	
			break;
		case Click:
			if (AIRBRIDGE_UNREAL_MODULE_POINTER)
				ANDROID_CORE_POINTER->ProcessClickOnSuccess(UUID);
			break;
		case Impression:
			if (AIRBRIDGE_UNREAL_MODULE_POINTER)
				ANDROID_CORE_POINTER->ProcessImpressionOnSuccess(UUID);
			break;
		case FetchDeviceUUID:
			if (AIRBRIDGE_UNREAL_MODULE_POINTER)
			{
				if (IsBroadcast)
				{
					ANDROID_CORE_POINTER->UnrealBroadcastDeviceUUID(ConvertString(Env, (jstring)Result));
				}
				else
				{
					ANDROID_CORE_POINTER->ProcessFetchDeviceUUIDOnSuccess(UUID, ConvertString(Env, (jstring)Result));
				}
			}
			break;
		case FetchAirbridgeGeneratedUUID:
			if (AIRBRIDGE_UNREAL_MODULE_POINTER)
			{
				if (IsBroadcast)
				{
					ANDROID_CORE_POINTER->UnrealBroadcastAirbridgeGeneratedUUID(ConvertString(Env, (jstring)Result));
				}
				else
				{
					ANDROID_CORE_POINTER->ProcessFetchAirbridgeGeneratedUUIDOnSuccess(UUID, ConvertString(Env, (jstring)Result));
				}
			}
			break;
		default:
			break;
		}
	}

	/*
	 * Class:     co_ab180_airbridge_ue_callback_AirbridgeOnFailureCallback
	 * Method:    invoke
	 * Signature: (Ljava/lang/Throwable;)V
	 */
	JNIEXPORT void JNICALL Java_co_ab180_airbridge_ue_callback_AirbridgeOnFailureCallback_invoke
	(JNIEnv *Env, jobject This, jthrowable Result)
	{
		if (Result == nullptr) {
			return;
		}
		// Find the Throwable class (java/lang/Throwable)
		jclass throwableClass = Env->GetObjectClass(Result);
		if (!throwableClass) return;
		
		// Get the getMessage method ID
		jmethodID getMessageMethodId = Env->GetMethodID(throwableClass, "getMessage", "()Ljava/lang/String;");
		if (!getMessageMethodId) return;
		
		// Call the getMessage method on the throwable
		jstring jMessage = (jstring) Env->CallObjectMethod(Result, getMessageMethodId);
		if (!jMessage) return;
		FString Message = ConvertString(Env, jMessage);

		// Get Method UUID
		FString UUID = GetCallbackUUID(Env, This);
		
		switch (GetCallbackType(Env, This)) {
		case HandleDeeplink:
			break;
		case HandleDeferredDeeplink:
			break;
		case Click:
			if (AIRBRIDGE_UNREAL_MODULE_POINTER)
				ANDROID_CORE_POINTER->ProcessClickOnFailure(UUID, Message);
			break;
		case Impression:
			if (AIRBRIDGE_UNREAL_MODULE_POINTER)
				ANDROID_CORE_POINTER->ProcessImpressionOnFailure(UUID, Message);
			break;
		case FetchDeviceUUID:
			if (AIRBRIDGE_UNREAL_MODULE_POINTER)
				ANDROID_CORE_POINTER->ProcessFetchDeviceUUIDOnFailure(UUID, Message);
			break;
		case FetchAirbridgeGeneratedUUID:
			if (AIRBRIDGE_UNREAL_MODULE_POINTER)
				ANDROID_CORE_POINTER->ProcessFetchAirbridgeGeneratedUUIDOnFailure(UUID, Message);
			break;
		default:
			break;
		}
	}

	/*
	 * Class:     co_ab180_airbridge_ue_callback_AirbridgeOnAttributionResultReceiveListener
	 * Method:    onAttributionResultReceived
	 * Signature: (Ljava/util/Map;)V
	 */
	JNIEXPORT void JNICALL Java_co_ab180_airbridge_ue_callback_AirbridgeOnAttributionResultReceiveListener_onAttributionResultReceived
	(JNIEnv *Env, jobject This, jobject AttributionResult)
	{
		TMap<FString, FString> AttributionResultData;
		if (AirbridgeJNI::JavaMapToUnrealTMap(Env, AttributionResult, AttributionResultData))
		{
			// Check if module is loaded
			if (AIRBRIDGE_UNREAL_MODULE_POINTER)
			{
				ANDROID_CORE_POINTER->ProcessOnAttributionResultReceived(AttributionResultData);
			}
			// If module is not loaded
			else
			{
				AirbridgeAndroidCore::ReceivedAttribution = AttributionResultData;
			}
		}
	}
}
#endif