// Copyright © 2023 ab180. All rights reserved.

#pragma once

#include "CoreMinimal.h"
#include "UObject/NoExportTypes.h"
#include "AirbridgeList.generated.h"

class UAirbridgeMap;

/**
 * A container class designed for handling and manipulating lists in Unreal Engine, with support for various data types.
 */
UCLASS()
class AIRBRIDGEUNREAL_API UAirbridgeList : public UObject
{
	GENERATED_BODY()

public:
	/**
	 * Creates an instance of the UAirbridgeList object.
	 * @return A pointer to a new instance of UAirbridgeList.
	 */
	/** @cond Doxygen_Suppress_Param */
	UFUNCTION(BlueprintCallable, Category = "Airbridge Container|List", meta = (DisplayName = "Create Airbridge List"))
	/** @endcond */
	static
	/** @cond Doxygen_Suppress_Param */
	UPARAM(DisplayName = "Airbridge List")
	/** @endcond */
	UAirbridgeList* CreateObject();

	/**
	 * Convert JSON Array to UAirbridgeList.
	 * @param JsonArray An array of shared pointer to a FJsonValue containing the data to be processed.
	 * @return A pointer to a new instance of UAirbridgeList.
	 */
	static UAirbridgeList* CreateObject(const TArray<TSharedPtr<FJsonValue>>& JsonArray);

#pragma region STRING_HANDLING

	/*
	 * 문자열 처리:
	 *
	 *		1. 기본적으로 Unreal에서는 FString 클래스를 사용하여 문자열을 처리한다.
	 *		
	 *		2. C-style 문자열 (e.g. "") 을 인자로 전달받는 경우 아래와 같은 함수를 통하여 처리한다.
	 *			:
	 *				UAirbridgeList* Add(const char* Value);
	 *				
	 *		3. TEXT 매크로를 사용하는 인자를 전달받은 경우 아래와 같은 함수를 통하여 처리한다.
	 *			:
	 *				#if PLATFORM_TCHAR_IS_CHAR16
	 *					UAirbridgeList* Add(const char16_t* Value);
	 *				#else
	 *					UAirbridgeList* Add(const wchar_t* Value);
	 *
	 *			===================================================================================
	 *			TEXT Macro
	 *			(/Users/Shared/Epic Games/UE_4.26/Engine/Source/Runtime/Core/Public/HAL/Platform.h)
	 *			===================================================================================
	 *			#if !defined(TEXT) && !UE_BUILD_DOCS
	 *				#if PLATFORM_TCHAR_IS_CHAR16
	 *					#define TEXT_PASTE(x) u ## x
	 *				#else
	 *					#define TEXT_PASTE(x) L ## x
	 *				#endif
	 *					#define TEXT(x) TEXT_PASTE(x)
	 *			#endif
	 *			===================================================================================
	 *
	 *		ps. 추가로 문제가 발생할 경우 가장 간단하게 해결할 수 있는 방법은 메서드 오버로드를 해제하는 것이다.
	 *///
	
	/**
	 * Adds a string value to the list.
	 * @param Value The string value to add.
	 * @return The UAirbridgeList pointer for method chaining.
	 */
	UAirbridgeList* Add(const FString& Value);
	
	/**
	 * Adds a string value to the list.
	 * @param Value The string value to add.
	 * @return The UAirbridgeList pointer for method chaining.
	 *
	 * @note A single 8-bit character.
	 */
	UAirbridgeList* Add(const char* Value);
	
#if PLATFORM_TCHAR_IS_CHAR16

	/**
	 * Adds a string value to the list.
	 * @param Value The string value to add.
	 * @return The UAirbridgeList pointer for method chaining.
	 *
	 * @note A single 16-bit character.
	 */
	UAirbridgeList* Add(const char16_t* Value);
	
#else

	/**
	 * Adds a string value to the list.
	 * @param Value The string value to add.
	 * @return The UAirbridgeList pointer for method chaining.
	 *
	 * @note A single wide-character.
	 */
	UAirbridgeList* Add(const wchar_t* Value);
	
#endif

#pragma endregion STRING_HANDLING
	// 	UAirbridgeList* Add({String} Value);

	/**
	 * Adds an integer value to the list.
	 * @param Value The integer value to add.
	 * @return The UAirbridgeList pointer for method chaining.
	 */
	UAirbridgeList* Add(const int32 Value);

	/**
	 * Adds a floating-point value to the list.
	 * @param Value The floating-point value to add.
	 * @return The UAirbridgeList pointer for method chaining.
	 */
	UAirbridgeList* Add(const double Value);

	/**
	 * Adds a boolean value to the list.
	 * @param Value The boolean value to add.
	 * @return The UAirbridgeList pointer for method chaining.
	 */
	UAirbridgeList* Add(const bool Value);

	/**
	 * Adds an UAirbridgeList value to the list.
	 * @param Value The UAirbridgeList value to add.
	 * @return The UAirbridgeList pointer for method chaining.
	 */
	UAirbridgeList* Add(UAirbridgeList* Value);

	/**
	 * Adds an UAirbridgeMap value to the list.
	 * @param Value The UAirbridgeMap value to add.
	 * @return The UAirbridgeList pointer for method chaining.
	 */
	UAirbridgeList* Add(UAirbridgeMap* Value);

	/**
	 * Adds a FJsonValue shared pointer value to the list.
	 * @param Value The FJsonValue shared pointer to add.
	 * @return The UAirbridgeList pointer for method chaining.
	 */
	UAirbridgeList* Add(const TSharedPtr<FJsonValue>& Value);
	
protected:
	/**
	 * Adds a string value to the list.
	 * @param Value The string value to add.
	 * @return The UAirbridgeList pointer for method chaining.
	 * @note This function is **Blueprint-only** and cannot be called from C++ code.
	 */
	/** @cond Doxygen_Suppress_Param */
	UFUNCTION(BlueprintCallable, Category = "Airbridge Container|List")
	UPARAM(DisplayName = "Return Target")
	/** @endcond */
	UAirbridgeList* AddString(const FString& Value);

	/**
	 * Adds an integer value to the list.
	 * @param Value The integer value to add.
	 * @return The UAirbridgeList pointer for method chaining.
	 * @note This function is **Blueprint-only** and cannot be called from C++ code.
	 */
	/** @cond Doxygen_Suppress_Param */
	UFUNCTION(BlueprintCallable, Category = "Airbridge Container|List")
	UPARAM(DisplayName = "Return Target")
	/** @endcond */
	UAirbridgeList* AddInteger(const int32 Value);

	/**
	 * Adds a floating-point value to the list.
	 * @param Value The floating-point value to add.
	 * @return The UAirbridgeList pointer for method chaining.
	 * @note This function is **Blueprint-only** and cannot be called from C++ code.
	 */
	/** @cond Doxygen_Suppress_Param */
	UFUNCTION(BlueprintCallable, Category = "Airbridge Container|List")
	UPARAM(DisplayName = "Return Target")
	/** @endcond */
	UAirbridgeList* AddFloat(const float Value);

	/**
	 * Adds a boolean value to the list.
	 * @param Value The boolean value to add.
	 * @return The UAirbridgeList pointer for method chaining.
	 * @note This function is **Blueprint-only** and cannot be called from C++ code.
	 */
	/** @cond Doxygen_Suppress_Param */
	UFUNCTION(BlueprintCallable, Category = "Airbridge Container|List")
	UPARAM(DisplayName = "Return Target")
	/** @endcond */
	UAirbridgeList* AddBoolean(const bool Value);

	/**
	 * Adds an UAirbridgeList value to the list.
	 * @param Value The UAirbridgeList value to add.
	 * @return The UAirbridgeList pointer for method chaining.
	 * @note This function is **Blueprint-only** and cannot be called from C++ code.
	 */
	/** @cond Doxygen_Suppress_Param */
	UFUNCTION(BlueprintCallable, Category = "Airbridge Container|List")
	UPARAM(DisplayName = "Return Target")
	/** @endcond */
	UAirbridgeList* AddList(UAirbridgeList* Value);

	/**
	 * Adds an UAirbridgeMap value to the list.
	 * @param Value The UAirbridgeMap value to add.
	 * @return The UAirbridgeList pointer for method chaining.
	 * @note This function is **Blueprint-only** and cannot be called from C++ code.
	 */
	/** @cond Doxygen_Suppress_Param */
	UFUNCTION(BlueprintCallable, Category = "Airbridge Container|List")
	UPARAM(DisplayName = "Return Target")
	/** @endcond */
	UAirbridgeList* AddMap(UAirbridgeMap* Value);
	
private:
	UPROPERTY()
	TArray<FString> ArrayOfString;
	
	UPROPERTY()
	TArray<int32> ArrayOfInteger;
	
	UPROPERTY()
	TArray<double> ArrayOfNumber;	// support double type for c++, support float type for blueprint
	
	UPROPERTY()
	TArray<bool> ArrayOfBoolean;
	
	UPROPERTY()
	TArray<UAirbridgeList*> ArrayOfList;

	UPROPERTY()
	TArray<UAirbridgeMap*> ArrayOfMap;

public:
#if PLATFORM_ANDROID
	void* CreateJavaObject() const;
#elif PLATFORM_IOS
	NSArray * ConvertToNSArray() const;
#endif
};